# R example for "Generally-altered, -inflated and -truncated regression,
# with application to heaped and seeped counts",
# by Yee and Ma,
# January 2021.


# Note:
# 1. Two R packages are needed below: "VGAM" and "VGAMdata".
# (a) Ideally the very latest version of VGAM is needed;
#     this is at my prerelease website described below.
#     But the latest CRAN version should be fine.
# (b) VGAMdata is on CRAN.



  options(digits = 5)



# ==================================================================
# Get the packages if necessary, and load them.

# Note: the very latest version is currently available at
# https://www.stat.auckland.ac.nz/~yee/VGAM/prerelease
# but VGAM version 1.1-5 is ideally needed here.




if (R.Version()$major < "4")
  stop("R version 4.0.0 or higher is needed")



if (!any(installed.packages()[, 'Package'] == "VGAM"))
  install.packages("VGAM")



library("VGAM")


if (packageVersion("VGAM") < "1.1.5")
  warning("you need to update 'VGAM' to at least version 1.1-5")

    

# Get VGAMdata installed if necessary -------------------------------
# If not yet installed then install it:

if (!any(installed.packages()[, 'Package'] == "VGAMdata"))
  install.packages("VGAMdata")



# ==================================================================
# GT-Expansion method on Maori pulse data.
# Section 5.2 of the paper.



data(xs.nz, package = "VGAMdata")


nona.xs.nz <-
  subset(xs.nz, !is.na(pulse) &
                !is.na(ethnicity) &
                ethnicity == "Maori")


dim(nona.xs.nz)


pulse4.df <-
  subset(nona.xs.nz, pulse %% 4 == 0, c(pulse))


pulse4.df <- transform(pulse4.df, pulse4 = pulse / 4)




# Table in the paper (starting point):
with(pulse4.df, table(pulse4))



dim(pulse4.df)


summary(pulse4.df)




# Moment estimator:
(m.hat <- with(pulse4.df, mean(pulse4) / var(pulse4)))
(round.m.hat <- round(m.hat))




# Apply the generally-truncated-expansion (GTE) method ,,,,,,,,,,,,,,
# allfits is all that is needed.
# Warnings can be ignored.
org1 <- with(pulse4.df, range(pulse4))  # Original data range
m.max <- 5  # Try multipliers 1:m.max
aics <- logliks <- numeric(m.max)
allfits <- vector("list", m.max)
names(allfits) <- names(aics) <-
names(logliks) <- as.character(1:m.max)
for (mux in seq(m.max)) {
  fit <- vglm(mux * pulse4 ~ offset(rep(log(mux), nrow(pulse4.df))),
              gaitpoisson(truncate = Trunc(org1, mux)),
              data = pulse4.df)
  allfits[[mux]] <- fit
  logliks[mux] <- logLik(fit)
  aics[mux] <- AIC(fit)
}




logliks
sort(logliks, decreasing = TRUE)  # Best to worst


round(sapply(allfits, coef), 3)
round(sqrt(sapply(allfits, vcov)) * 1000, 1)



# Reconstructed response ,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
mux.choose <- as.vector(which.max(logliks))  # 3 is better than 4
mux.choose


Coef(allfits[[mux.choose]]) * mux.choose  # Point estimate
exp(confint(allfits[[mux.choose]])) * mux.choose  # Approx 95% CI



# Graphically compare the multipliers.
plot(logliks, col = "blue", type = "b",
     main = "GTE method applied to pulse4.df",
     xlab = "Multiplier")




# ==================================================================
# Plot the results.



pulse4.df <-
  transform(pulse4.df, y.use = pulse4 * mux.choose)


Tvec <- with(pulse4.df, Trunc(range(pulse4), mux.choose))


# The best fit, for plotting in (b)
fit3 <- vglm(y.use ~ 1,
             gaitpoisson(truncate = Tvec),
             data = pulse4.df, trace = TRUE, crit = "coef")


pdf.est3 <- dgaitpois(as.numeric(names(table(depvar(fit3)))),
                      lambda.p = Coef(fit3),
                      truncate = Tvec)





par(mfrow = c(1, 2), las = 0, xpd = FALSE)


# Plot (a) ,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,

spikeplot(with(pulse4.df, pulse4), main = "(a)",
          las = 1, ylim = c(0, 0.2), capped = TRUE,
          xlab = "Beats", ylab = "Proportion")




# Poisson is inferior
badfit <- vglm(pulse4 ~ 1, poissonff, data = pulse4.df, trace = TRUE)
Ty <- as.numeric(names(table(depvar(badfit))))
pdf.est1 <- dpois(Ty, Coef(badfit))
lines(Ty, pdf.est1, col = "red",
      type = "o", lty = "dashed", cex = 0.5, lwd = 0.50)


# Add in the good fit
lines(Ty, pdf.est3, col = "pink2",
      type = "o", lty = "solid", cex = 0.5, lwd = 1.25)





# Plot (b) ,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,

spikeplot(with(pulse4.df, y.use), main = "(b)",
          las = 1, ylim = c(0, 0.2), capped = TRUE,
          xlab = expression(3 %*% Beats), ylab = "")



plotdgait(fit3, new.plot = FALSE, type.plot = "o", cex.t = 0.5,
          col.p = "pink2",
          lty.p = "solid", lwd.p = 1.25, cex.p = 0.5)






# ==================================================================
# ==================================================================
# The following code chunk illustrates the generation of underdispersed
# random variates based on a common standard distribution (e.g., Poisson)
# without the need of an exotic underdispersed distribution such
# as the Conway--Maxwell--Poisson (CMP).
# It applies the GTE method in reverse.



set.seed(1)
n <- 10000
lambda <- 10
tvec <- seq(1, lambda + 5*lambda^0.5, by=2)  # Odd values over the range

y <- rgaitpois(n, lambda, truncate = tvec) / 2

# table(y)
c(mean = mean(y), var = var(y), VMR = var(y) / mean(y))


# Plot the distributions side-by-side. The Poisson distribution
# has the same mean.
par(mfrow = c(1, 1))
x <- 0:12
plot(table(y) / n, las = 1, col = "black", xlim = range(x),
     main = "Black = underdispersed; blue dashed = Poisson",
     ylab = "Probability/proportion")
lines(x + 0.2, dpois(x, lambda / 2), col = "blue",
      lty = "dashed", type = "h")



# ==================================================================







